//--------------------------------------------------------------
//  IIR フィルタを利用する直交信号発生器のクラス
//
//  2022/10/26, Copyright (c) 2022 MIKAMI, Naoki
//--------------------------------------------------------------

#include "mbed.h"

#ifndef QUAD_PHASE_OSC_IIR_HPP
#define QUAD_PHASE_OSC_IIR_HPP

namespace Mikami
{
    class QuadOscIir
    {
    public:
        // コンストラクタ
        //      f0  周波数
        //      fs  標本化周波数
        //      f0, fs は同じ単位
        QuadOscIir(float f0, float fs) : TS_(1.0f/fs)
        { Set(f0); }

        // sin/cos 発生
        void Generate(float &sinX, float &cosX)
        {
            float vn = a1_*vn1_ - vn2_;
            sinX = b1S_*vn1_;
            cosX = vn + b1C_*vn1_;

            vn2_ = vn1_;    // v[n-2] ← v[n-1]
            vn1_ = vn;      // v[n-1] ← v[n]
        }

        // 出力周波数の設定，周波数を変更し再スタートする場合も使える
        //      f0：周波数，単位：Hz
        void Set(float f0)
        {
            static const float PI2 = 6.283185f;
            a1_ = 2.0f*cosf(PI2*f0*TS_);
            b1S_ = sinf(PI2*f0*TS_);
            b1C_ = -cosf(PI2*f0*TS_);
            vn1_ = 1.0f;
            vn2_ = 0.0f;
        }

    private:
        const float TS_;        // 標本化間隔
        float a1_, b1S_, b1C_;
        float vn1_, vn2_;

        // コピー･コンストラクタ，代入演算子の禁止のため
        QuadOscIir(const QuadOscIir&);
        QuadOscIir& operator=(const QuadOscIir&);
    };
}
#endif  // QUAD_PHASE_OSC_IIR_HPP
