//----------------------------------------------------------------------
//  ヒルベルト変換フィルタを利用する周波数シフタ，100 Hz 高い方へシフト
//      ヒルベルト変換フィルタにはリングバッファ方式を使用
//      ヒルベルト変換フィルタの通過域：0.2 ～ 21.85 kHz
//      DC 除去用の HPF なし
//
//  2023/10/02, Copyright (c) 2023 MIKAMI, Naoki
//----------------------------------------------------------------------

#include "MultirateLiPh.hpp"
#include "HilbertTransformer.hpp"   // リングバッファ方式のヒルベルト変換フィルタ
#include "coefsHilber202.hpp"
#include "QuadOscIir.hpp"
#include "Biquad.hpp"
#include "DC_Cut_Coefficients.hpp"
#pragma diag_suppress 870   // マルチバイト文字使用の警告抑制のため
using namespace Mikami;

const float FS_ = 44.1f;        // 入力の標本化周波数： 44.1 kHz
MultirateLiPh myAdDa_(FS_);     // 出力標本化周波数を４倍にするオブジェクト

Hilbert ht_(ORDER_, HM_);       // ヒルベルト変換フィルタ
Biquad dcCut(C1_);              // 直流分除去フィルタ
QuadOscIir osc_(0.1f, FS_);     // 直交信号発生器，シフトする周波数：+0.1 kHz

bool on_ = true;

void AdcIsr()
{
    float xn = myAdDa_.Input();   // 入力
    xn += 0.1f;
//    xn = dcCut.Execute(G0_*xn); // 直流分除去

    float ynI, ynQ;
    ht_.Execute(xn, ynI, ynQ);  // ヒルベルト変換フィルタ実行

    float sinX, cosX;
    osc_.Generate(sinX, cosX);

    float yn = ynI*cosX - ynQ*sinX;
    
    if (on_) myAdDa_.Output(yn);    // 出力（周波数シフタの出力）
    else     myAdDa_.Output(xn);    // 出力（入力そのまま）
}

int main()
{
    setbuf(stdout, NULL);   // printf() でバッファ無しの設定
    printf("\r\nヒルベルト変換器を利用する周波数シフタを実行します\r\n");
    printf("DC 除去用の HPF なし\r\n");
    printf("周波数シフタの ON/OFF はキーボードからコントロールできます\r\n");
    printf("起動時は ON の状態です\r\n");
    printf("'y'：ON, 'n'：Off\r\n");

    myAdDa_.Start(&AdcIsr); // 標本化を開始する

    while (true)
    {
        char yesNo = getchar();
        printf("%c", yesNo);
        on_ = (yesNo == 'y') ? true : false;
    }
}
