//----------------------------------------------------------------------
//  ヒルベルト変換フィルタ
//      ヒルベルト変換フィルタはリングバッファを使用
//      DA 出力はアップサンプリングしない
//      ヒルベルト変換フィルタの通過域：0.2 ～ 21.85 kHz
//      A2：I 信号，D13：Q信号
//
//  D13 はマイコン･ボードの LED が接続されているため，入力信号の振幅が
//  ある程度以上になると，波形の＋側がクリップされる
//
//  2023/10/02, Copyright (c) 2023 MIKAMI, Naoki
//----------------------------------------------------------------------

#include "DSP_AdcIntr.hpp"  // ADC 用，割り込み使用
#include "DSP_DacDual.hpp"  // DAC 用
#include "HilbertTransformer.hpp"
#include "coefsHilber202.hpp"
#pragma diag_suppress 870   // マルチバイト文字使用の警告抑制のため
using namespace Mikami;

const float FS_ = 44.1f;        // 入力の標本化周波数： 44.1 kHz
DspAdcIntr myAdc_(FS_, A1);     // AD 変換器, 入力端子: A1
DspDacDual myDac_;              // DA 変換器, 出力端子: A2, D13

Hilbert ht_(ORDER_, HM_);       // ヒルベルト変換フィルタ

// ADC 変換終了割込みに対する割込みサービス･ルーチン
void AdcIsr()
{
    float xn = myAdc_.Read();   // 入力

    float ynI, ynQ;
    ht_.Execute(xn, ynI, ynQ);  // ヒルベルト変換フィルタ実行

    myDac_.Write(ynI, ynQ);     // 出力 
}

int main()
{
    printf("\r\nヒルベルト変換フィルタを実行します\r\n");

    // ADC 変換終了割込みに対する割込みサービス･ルーチンを割り当てる
    myAdc_.SetIntrVec(&AdcIsr);

    while (true) {}
}
